//********************************************************************
//*** Exceptions/Errno.cc
//*** Copyright (c) 2001 by Markus Winand <mws@fatalmind.com>
//*** $Id: Errno.cc,v 1.16 2003/05/19 20:33:59 mws Exp $
//********************************************************************


#include <stdio.h>
#include "OS_Errno.hh"
#include "Errno.hh"
#include <string.h>
#include <memory>
//#include <errno.h>

using namespace std;
namespace fatalmind {
void Errno::Throw(const string& fmt) {
        int err = errno;
        switch (err) {
                case OS_E2BIG: throw Errno::E2BIG(fmt);
                case OS_EACCES: throw Errno::EACCES(fmt);
                case OS_EAGAIN: throw Errno::EAGAIN(fmt);
                case OS_EBADF: throw Errno::EBADF(fmt);
                case OS_EBUSY: throw Errno::EBUSY(fmt);
                case OS_ECHILD: throw Errno::ECHILD(fmt);
                case OS_EDEADLK: throw Errno::EDEADLK(fmt);
                case OS_EDOM: throw Errno::EDOM(fmt);
                case OS_EEXIST: throw Errno::EEXIST(fmt);
                case OS_EFAULT: throw Errno::EFAULT(fmt);
                case OS_EFBIG: throw Errno::EFBIG(fmt);
                case OS_EINTR: throw Errno::EINTR(fmt);
                case OS_EINVAL: throw Errno::EINVAL(fmt);
                case OS_EIO: throw Errno::EIO(fmt);
                case OS_EISDIR: throw Errno::EISDIR(fmt);
                case OS_EMFILE: throw Errno::EMFILE(fmt);
                case OS_EMLINK: throw Errno::EMLINK(fmt);
                case OS_ENAMETOOLONG: throw Errno::ENAMETOOLONG(fmt);
                case OS_ENFILE: throw Errno::ENFILE(fmt);
                case OS_ENODEV: throw Errno::ENODEV(fmt);
                case OS_ENOENT: throw Errno::ENOENT(fmt);
                case OS_ENOEXEC: throw Errno::ENOEXEC(fmt);
                case OS_ENOLCK: throw Errno::ENOLCK(fmt);
                case OS_ENOMEM: throw Errno::ENOMEM(fmt);
                case OS_ENOSPC: throw Errno::ENOSPC(fmt);
                case OS_ENOSYS: throw Errno::ENOSYS(fmt);
                case OS_ENOTDIR: throw Errno::ENOTDIR(fmt);
                case OS_ENOTEMPTY: throw Errno::ENOTEMPTY(fmt);
                case OS_ENOTTY: throw Errno::ENOTTY(fmt);
                case OS_ENXIO: throw Errno::ENXIO(fmt);
                case OS_EPERM: throw Errno::EPERM(fmt);
                case OS_EPIPE: throw Errno::EPIPE(fmt);
                case OS_ERANGE: throw Errno::ERANGE(fmt);
                case OS_EROFS: throw Errno::EROFS(fmt);
                case OS_ESPIPE: throw Errno::ESPIPE(fmt);
                case OS_ESRCH: throw Errno::ESRCH(fmt);
                case OS_EXDEV: throw Errno::EXDEV(fmt);
                case OS_ETIMEDOUT: throw Errno::ETIMEDOUT(fmt);
        }
        
        throw Errno(err,fmt);
}

void Errno::ThrowOnError(int err) {
	if (err != 0) {
		Throw(err);
	}
}

void Errno::Throw(int err) {
	if (err == 0) {
		err = errno;
	}
	switch (err) {
		case OS_E2BIG: throw Errno::E2BIG();
		case OS_EACCES: throw Errno::EACCES();
		case OS_EAGAIN: throw Errno::EAGAIN();
		case OS_EBADF: throw Errno::EBADF();
		case OS_EBUSY: throw Errno::EBUSY();
		case OS_ECHILD: throw Errno::ECHILD();
		case OS_EDEADLK: throw Errno::EDEADLK();
		case OS_EDOM: throw Errno::EDOM();
		case OS_EEXIST: throw Errno::EEXIST();
		case OS_EFAULT: throw Errno::EFAULT();
		case OS_EFBIG: throw Errno::EFBIG();
		case OS_EINTR: throw Errno::EINTR();
		case OS_EINVAL: throw Errno::EINVAL();
		case OS_EIO: throw Errno::EIO();
		case OS_EISDIR: throw Errno::EISDIR();
		case OS_EMFILE: throw Errno::EMFILE();
		case OS_EMLINK: throw Errno::EMLINK();
		case OS_ENAMETOOLONG: throw Errno::ENAMETOOLONG();
		case OS_ENFILE: throw Errno::ENFILE();
		case OS_ENODEV: throw Errno::ENODEV();
		case OS_ENOENT: throw Errno::ENOENT();
		case OS_ENOEXEC: throw Errno::ENOEXEC();
		case OS_ENOLCK: throw Errno::ENOLCK();
		case OS_ENOMEM: throw Errno::ENOMEM();
		case OS_ENOSPC: throw Errno::ENOSPC();
		case OS_ENOSYS: throw Errno::ENOSYS();
		case OS_ENOTDIR: throw Errno::ENOTDIR();
		case OS_ENOTEMPTY: throw Errno::ENOTEMPTY();
		case OS_ENOTTY: throw Errno::ENOTTY();
		case OS_ENXIO: throw Errno::ENXIO();
		case OS_EPERM: throw Errno::EPERM();
		case OS_EPIPE: throw Errno::EPIPE();
		case OS_ERANGE: throw Errno::ERANGE();
		case OS_EROFS: throw Errno::EROFS();
		case OS_ESPIPE: throw Errno::ESPIPE();
		case OS_ESRCH: throw Errno::ESRCH();
		case OS_EXDEV: throw Errno::EXDEV();
		case OS_ETIMEDOUT: throw Errno::ETIMEDOUT();
	}
	
	throw Errno(err);
}

Errno::Errno(const int err): Exception(), _errno(err==0?errno:err) {
}

Errno::Errno(const int err, const string& fmt): Exception(fmt), _errno(err==0?errno:err) {
}

Errno::~Errno() {
}

int Errno::os_errno() const throw(){
	return _errno;
}
/*
class Errno Errno::mk_new(int err=0) {
	switch (err) {
		case OS_EPERM: return Errno::EPERM();
	}
	printf("ERROR\n");
	return Errno();
}*/

const string Errno::message() const throw(){
	string ret(strerror(_errno));
	return ret;
}

Errno::E2BIG::E2BIG(): Errno(OS_E2BIG) {}
Errno::E2BIG::E2BIG(const string& fmt): Errno(OS_E2BIG, fmt) {}
Errno::E2BIG::~E2BIG() {}
Errno::EACCES::EACCES(): Errno(OS_EACCES) {}
Errno::EACCES::EACCES(const string& fmt): Errno(OS_EACCES, fmt) {}
Errno::EACCES::~EACCES() {}
Errno::EAGAIN::EAGAIN(): Errno(OS_EAGAIN) {}
Errno::EAGAIN::EAGAIN(const string& fmt): Errno(OS_EAGAIN, fmt) {}
Errno::EAGAIN::~EAGAIN() {}
Errno::EBADF::EBADF(): Errno(OS_EBADF) {}
Errno::EBADF::EBADF(const string& fmt): Errno(OS_EBADF, fmt) {}
Errno::EBADF::~EBADF() {}
Errno::EBUSY::EBUSY(): Errno(OS_EBUSY) {}
Errno::EBUSY::EBUSY(const string& fmt): Errno(OS_EBUSY, fmt) {}
Errno::EBUSY::~EBUSY() {}
Errno::ECHILD::ECHILD(): Errno(OS_ECHILD) {}
Errno::ECHILD::ECHILD(const string& fmt): Errno(OS_ECHILD, fmt) {}
Errno::ECHILD::~ECHILD() {}
Errno::EDEADLK::EDEADLK(): Errno(OS_EDEADLK) {}
Errno::EDEADLK::EDEADLK(const string& fmt): Errno(OS_EDEADLK, fmt) {}
Errno::EDEADLK::~EDEADLK() {}
Errno::EDOM::EDOM(): Errno(OS_EDOM) {}
Errno::EDOM::EDOM(const string& fmt): Errno(OS_EDOM, fmt) {}
Errno::EDOM::~EDOM() {}
Errno::EEXIST::EEXIST(): Errno(OS_EEXIST) {}
Errno::EEXIST::EEXIST(const string& fmt): Errno(OS_EEXIST, fmt) {}
Errno::EEXIST::~EEXIST() {}
Errno::EFAULT::EFAULT(): Errno(OS_EFAULT) {}
Errno::EFAULT::EFAULT(const string& fmt): Errno(OS_EFAULT, fmt) {}
Errno::EFAULT::~EFAULT() {}
Errno::EFBIG::EFBIG(): Errno(OS_EFBIG) {}
Errno::EFBIG::EFBIG(const string& fmt): Errno(OS_EFBIG, fmt) {}
Errno::EFBIG::~EFBIG() {}
Errno::EINTR::EINTR(): Errno(OS_EINTR) {}
Errno::EINTR::EINTR(const string& fmt): Errno(OS_EINTR, fmt) {}
Errno::EINTR::~EINTR() {}
Errno::EINVAL::EINVAL(): Errno(OS_EINVAL) {}
Errno::EINVAL::EINVAL(const string& fmt): Errno(OS_EINVAL, fmt) {}
Errno::EINVAL::~EINVAL() {}
Errno::EIO::EIO(): Errno(OS_EIO) {}
Errno::EIO::EIO(const string& fmt): Errno(OS_EIO, fmt) {}
Errno::EIO::~EIO() {}
Errno::EISDIR::EISDIR(): Errno(OS_EISDIR) {}
Errno::EISDIR::EISDIR(const string& fmt): Errno(OS_EISDIR, fmt) {}
Errno::EISDIR::~EISDIR() {}
Errno::EMFILE::EMFILE(): Errno(OS_EMFILE) {}
Errno::EMFILE::EMFILE(const string& fmt): Errno(OS_EMFILE, fmt) {}
Errno::EMFILE::~EMFILE() {}
Errno::EMLINK::EMLINK(): Errno(OS_EMLINK) {}
Errno::EMLINK::EMLINK(const string& fmt): Errno(OS_EMLINK, fmt) {}
Errno::EMLINK::~EMLINK() {}
Errno::ENAMETOOLONG::ENAMETOOLONG(): Errno(OS_ENAMETOOLONG) {}
Errno::ENAMETOOLONG::ENAMETOOLONG(const string& fmt): Errno(OS_ENAMETOOLONG, fmt) {}
Errno::ENAMETOOLONG::~ENAMETOOLONG() {}
Errno::ENFILE::ENFILE(): Errno(OS_ENFILE) {}
Errno::ENFILE::ENFILE(const string& fmt): Errno(OS_ENFILE, fmt) {}
Errno::ENFILE::~ENFILE() {}
Errno::ENODEV::ENODEV(): Errno(OS_ENODEV) {}
Errno::ENODEV::ENODEV(const string& fmt): Errno(OS_ENODEV, fmt) {}
Errno::ENODEV::~ENODEV() {}
Errno::ENOENT::ENOENT(): Errno(OS_ENOENT) {}
Errno::ENOENT::ENOENT(const string& fmt): Errno(OS_ENOENT, fmt) {}
Errno::ENOENT::~ENOENT() {}
Errno::ENOEXEC::ENOEXEC(): Errno(OS_ENOEXEC) {}
Errno::ENOEXEC::ENOEXEC(const string& fmt): Errno(OS_ENOEXEC, fmt) {}
Errno::ENOEXEC::~ENOEXEC() {}
Errno::ENOLCK::ENOLCK(): Errno(OS_ENOLCK) {}
Errno::ENOLCK::ENOLCK(const string& fmt): Errno(OS_ENOLCK, fmt) {}
Errno::ENOLCK::~ENOLCK() {}
Errno::ENOMEM::ENOMEM(): Errno(OS_ENOMEM) {}
Errno::ENOMEM::ENOMEM(const string& fmt): Errno(OS_ENOMEM, fmt) {}
Errno::ENOMEM::~ENOMEM() {}
Errno::ENOSPC::ENOSPC(): Errno(OS_ENOSPC) {}
Errno::ENOSPC::ENOSPC(const string& fmt): Errno(OS_ENOSPC, fmt) {}
Errno::ENOSPC::~ENOSPC() {}
Errno::ENOSYS::ENOSYS(): Errno(OS_ENOSYS) {}
Errno::ENOSYS::ENOSYS(const string& fmt): Errno(OS_ENOSYS, fmt) {}
Errno::ENOSYS::~ENOSYS() {}
Errno::ENOTDIR::ENOTDIR(): Errno(OS_ENOTDIR) {}
Errno::ENOTDIR::ENOTDIR(const string& fmt): Errno(OS_ENOTDIR, fmt) {}
Errno::ENOTDIR::~ENOTDIR() {}
Errno::ENOTEMPTY::ENOTEMPTY(): Errno(OS_ENOTEMPTY) {}
Errno::ENOTEMPTY::ENOTEMPTY(const string& fmt): Errno(OS_ENOTEMPTY, fmt) {}
Errno::ENOTEMPTY::~ENOTEMPTY() {}
Errno::ENOTTY::ENOTTY(): Errno(OS_ENOTTY) {}
Errno::ENOTTY::ENOTTY(const string& fmt): Errno(OS_ENOTTY, fmt) {}
Errno::ENOTTY::~ENOTTY() {}
Errno::ENXIO::ENXIO(): Errno(OS_ENXIO) {}
Errno::ENXIO::ENXIO(const string& fmt): Errno(OS_ENXIO, fmt) {}
Errno::ENXIO::~ENXIO() {}
Errno::EPERM::EPERM(): Errno(OS_EPERM) {}
Errno::EPERM::EPERM(const string& fmt): Errno(OS_EPERM, fmt) {}
Errno::EPERM::~EPERM() {}
Errno::EPIPE::EPIPE(): Errno(OS_EPIPE) {}
Errno::EPIPE::EPIPE(const string& fmt): Errno(OS_EPIPE, fmt) {}
Errno::EPIPE::~EPIPE() {}
Errno::ERANGE::ERANGE(): Errno(OS_ERANGE) {}
Errno::ERANGE::ERANGE(const string& fmt): Errno(OS_ERANGE, fmt) {}
Errno::ERANGE::~ERANGE() {}
Errno::EROFS::EROFS(): Errno(OS_EROFS) {}
Errno::EROFS::EROFS(const string& fmt): Errno(OS_EROFS, fmt) {}
Errno::EROFS::~EROFS() {}
Errno::ESPIPE::ESPIPE(): Errno(OS_ESPIPE) {}
Errno::ESPIPE::ESPIPE(const string& fmt): Errno(OS_ESPIPE, fmt) {}
Errno::ESPIPE::~ESPIPE() {}
Errno::ESRCH::ESRCH(): Errno(OS_ESRCH) {}
Errno::ESRCH::ESRCH(const string& fmt): Errno(OS_ESRCH, fmt) {}
Errno::ESRCH::~ESRCH() {}
Errno::EXDEV::EXDEV(): Errno(OS_EXDEV) {}
Errno::EXDEV::EXDEV(const string& fmt): Errno(OS_EXDEV, fmt) {}
Errno::EXDEV::~EXDEV() {}
Errno::ETIMEDOUT::ETIMEDOUT(): Errno(OS_ETIMEDOUT) {}
Errno::ETIMEDOUT::ETIMEDOUT(const string& fmt): Errno(OS_ETIMEDOUT, fmt) {}
Errno::ETIMEDOUT::~ETIMEDOUT() {}
}
