//********************************************************************
//*** ResourcePool/mysql/SQLSelect.hh
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelect.cc,v 1.16 2005/01/06 16:27:58 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */


#include "SQLSelect.hh"

#include "lex.mysql_bind.hh"

#ifndef INCLUDED_EXCEPTIONS_HH
#include "Exceptions/Exception.hh"
#endif

#ifndef RESOURCEPOOL_MYSQL_SQLTYPE_HH
#include "SQLType.hh"
#endif

#ifndef RESOURCEPOOL_MYSQL_MYSQLEXCEPTION_HH
#include "MysqlException.hh"
#endif

#include "ResourcePool/SQL/BindOut.cc"
//#include "SQLType.cc"

#include <mysql/mysql.h>
#include <sstream>

namespace fatalmind {
namespace mysql {

SQLSelect::SQLSelect(const std::string& SQL)
: _SQL(SQL)
, _rs(0)
, _num_fields(0)
{
}

SQLSelect::~SQLSelect()
{
}

void SQLSelect::finish() {
	if (_rs) {
		while (mysql_fetch_row(_rs)) {
			// empty buffer (whow, thats really required)
		}
		close_rs();
	}
}

void SQLSelect::close_rs() {
	mysql_free_result(_rs);
	_rs = 0;
}

void SQLSelect::execute(MYSQL& dbh)
{
	finish();
	mysql_bind boundsql(dbh, _SQL, _args);
	if (int rc = mysql_real_query(&dbh
		, boundsql.getSQL().c_str()
		, boundsql.getSQL().length())
	) {
		MysqlException::throwit(rc, mysql_error(&dbh), mysql_errno(&dbh), _SQL);
	}
	
	if ((_rs = mysql_use_result(&dbh)) == NULL) {
		MysqlException::throwit(0, mysql_error(&dbh), mysql_errno(&dbh), _SQL);
	}
	_num_fields = mysql_num_fields(_rs);

	internal::registerFinish fin(*this);
	while (fetch() && process()) {
	}
}

bool SQLSelect::fetch() {
	if (_rs) {
		// TODO: improve exception safty by introducing an object for _rs
		try {
			MYSQL_ROW row;
			row = mysql_fetch_row(_rs);
			if (row) {
				unsigned long *lengths;
				lengths = mysql_fetch_lengths(_rs);
				if (_outparam.size() < _num_fields) {
					std::ostringstream os;
					os << "bindout not called for all " << _num_fields << " columns.";
					throw NFOMysqlException(os.str(), 98, _SQL);
				}
				for (unsigned int i = 0; i < _num_fields; ++i) {
					if (_outparam[i].get() != 0) {
						SQLRefType& op = *_outparam[i];
						op.assign(row[i], lengths[i]);
					} else {
						std::ostringstream os;
						os << "bindout not called for column " << i;
						throw NFOMysqlException(os.str(), 98, _SQL);
					}
				}
			} else {
				close_rs();
				return false;
			}
		} catch (...) {
			close_rs();
			throw;
		}
	} else {
		return false;
	}
	return true;
}

} // namespace mysql

template class
SQL::Bind<mysql::SQLType, mysql::SQLTypeImpl>;

template class
SQL::BindOut<mysql::SQLRefType, mysql::SQLRefTypeImpl>;

} // namespace fatalmind
