//********************************************************************
//*** ResourcePool/oracle/CommandCommon.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: CommandCommon.cc,v 1.29 2005/01/06 16:28:00 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_COMMANDCOMMON_HH
#include "CommandCommon.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCE_HH
#include "Resource.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTIONFACTORY_HH
#include "OracleExceptionFactory.hh"
#endif

#include "oracleType.cc"
#include "ResourcePool/SQL/Bind.cc"

namespace fatalmind {
namespace oracle {

template<class TM>
CommandCommon<TM>::CommandCommon(const std::string& SQL) 
: _SQL(SQL)
, _sth()
{
}

template<class TM>
CommandCommon<TM>::~CommandCommon() {
}

template<class TM>
void CommandCommon<TM>::execute(oracleHandlesWithCache<TM>& hand) {
	StatementCache sthcache;
	_execute(hand, sthcache);
}


template<class TM>
void CommandCommon<TM>::prepare(oracleHandles& hand, StatementCache& sthcache, const oracleStatement& oraSQL) {
	_sth = sthcache.open(oraSQL, true);
	_sth->prepare(hand);
}

template<class TM>
void CommandCommon<TM>::real_bind(oracleHandles& hand) {
	sword ret;
	_argst::const_iterator end = _args.end();
	int pos = 0;
	for (_argst::const_iterator i = _args.begin(); i != end; ++i) {
		++pos;
		OCIBind  *bnd1p = NULL;
		if ((*i).get() != 0) {
			if ((ret = OCIBindByPos(_sth->get()
				, &bnd1p
				, hand.getError()
				, pos 
				, (*i)->ptr()
				, (*i)->size()
				, (*i)->type()
				, (*i)->indptr()
				, (ub2 *)0
				, (ub2 *)0
				, 0
				, (ub4 *)0
				, OCI_DEFAULT)
			) != OCI_SUCCESS) {
				OracleExceptionFactory::throwit("OCIBindByPos", hand, ret, _SQL);
			}
		} else {
			OracleExceptionFactory::throwit("OCIBindByPos", "ORA", 1008, "not all params bound", _SQL);
		} 
	}
}

template<class TM>
const std::string& CommandCommon<TM>::getSQL() const {
	return _SQL;
}

template class CommandCommon<DefaultThreadedModel >;


} // namespace oracle

template class
SQL::Bind<oracle::oracleType, oracle::oracleTypeImpl>;

//template class
//Command<oracleHandlesWithCache<DefaultThreadedModel > >;


} // namespace fatalmind

