//********************************************************************
//*** ResourcePool/oracle/StatementHandle.hh
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: OCIHandle.cc,v 1.18 2005/01/06 16:28:00 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_OCIHANDLE_HH
#include "OCIHandle.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTIONFACTORY_HH
#include "OracleExceptionFactory.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCE_HH
#include "Resource.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEBIND_HH
#include "lex.oracle_bind.hh"
#endif

#ifndef OCI_ORACLE
#include <oci.h>
#endif

#include <ostream>

namespace fatalmind {
namespace oracle {

template<typename HTYPE, int HCONST>
OCIHandle<HTYPE, HCONST>::OCIHandle(const OCIEnvHandle& env) 
{
	sword ret;
	if ((ret = OCIHandleAlloc(env.get()
		, (dvoid**)  &_h
		, HCONST
		, 0
		, (dvoid **)0)
	) != OCI_SUCCESS) {
		OracleExceptionFactory::throwit("OCIHandleAlloc/", env, ret, "");
	}
}

template<typename HTYPE, int HCONST>
OCIHandle<HTYPE, HCONST>::OCIHandle(const OCIEnv* env) 
{
	sword ret;
	if ((ret = OCIHandleAlloc(env
		, (dvoid**)  &_h
		, HCONST
		, 0
		, (dvoid **)0)
	) != OCI_SUCCESS) {
		OracleExceptionFactory::throwit("OCIHandleAlloc/", env, ret, "");
	}
}

template<typename HTYPE, int HCONST>
OCIHandle<HTYPE, HCONST>::~OCIHandle()
{
	OCIHandleFree((dvoid *)_h, (ub4) HCONST);
}

template<typename HTYPE, int HCONST>
HTYPE* OCIHandle<HTYPE, HCONST>::get() const
{
	return _h;
}

template class OCIHandle<OCIError,   OCI_HTYPE_ERROR>;
template class OCIHandle<OCIServer,  OCI_HTYPE_SERVER>;
template class OCIHandle<OCISvcCtx,  OCI_HTYPE_SVCCTX>;
template class OCIHandle<OCISession, OCI_HTYPE_SESSION>;

/*
OCIStatementHandle::OCIStatementHandle(const std::string& sql)
: _SQL()
, _sth()
, _prepare(0)
, _force_prepare(0)
{
	oracle_bind oracle(sql, _SQL);
}
*/

OCIStatementHandle::OCIStatementHandle(const oracleStatement& orasql)
: _SQL(orasql.getSQL())
, _sth()
, _prepare(0)
, _force_prepare(0)
{
}

void OCIStatementHandle::prepare(oracleHandles& hand) {
	++_prepare;
	if (_sth.get() == 0) {
		// if it's not prepared yet -> prepare it
		force_prepare(hand);
	} else {
		// if it is prepared, check if it is prepared against
		// the right OCIEnv
		OCIEnv* env;
		sword ret;
		if ((ret = OCIAttrGet(_sth->get()
			, OCI_HTYPE_STMT
			, &env
			, 0
			, OCI_ATTR_ENV
			, hand.getError())
		) != OCI_SUCCESS) {
			OracleExceptionFactory::throwit("OCIAttrGet", hand, ret, "");
		}
		if (env != hand.getEnv()) {
			// if not -> prepare new
			force_prepare(hand);
		}
	}
}

OCIStmt* OCIStatementHandle::get() const 
{
	return _sth->get();
}

void OCIStatementHandle::force_prepare(oracleHandles& hand) {
	++_force_prepare;
	_sth.reset(new Stmt_t(hand.getEnv()));
	sword ret;
	/* prepare statement */
	if ((ret = OCIStmtPrepare(_sth->get()
		, hand.getError()
		, (OraText*)_SQL.c_str()
		, (ub4) _SQL.length()
		, (ub4) OCI_NTV_SYNTAX
		, (ub4) OCI_DEFAULT))
	) {
		OracleExceptionFactory::throwit("OCIStmtPrepare", hand, ret, "");
	}
}

unsigned int OCIStatementHandle::getEfficienty() const 
{
	return _prepare;
	// the _prepare is the efficiency, _force_prepare should be always 1
	// _prepare means the more the better
}

std::ostream& operator<<(std::ostream& s, const OCIStatementHandle& rho) 
{
	return s << '\'' << rho._SQL << '\'' 
		<< " [" << rho._force_prepare << '/' << rho._prepare << ']' << std::endl;
}

} // namespace oracle
} // namespace fatalmind
