//********************************************************************
//*** ResourcePool/oracle/SQLSelectRow.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelectRow.cc,v 1.32 2005/01/06 16:28:00 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_SQLSELECTROW_HH
#include "SQLSelectRow.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCE_HH
#include "Resource.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLETYPE_HH
#include "oracleType.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEREFTYPE_HH
#include "oracleRefType.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTIONFACTORY_HH
#include "OracleExceptionFactory.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_SELECTLISTMETADATA_HH
#include "SelectListMetaData.hh"
#endif

#include "ResourcePool/SQL/BindOut.cc"

#include "oracleRefType.cc"

#include "SelectListMetaDataCache.hh"

#ifndef RESOURCEPOOL_ORACLE_COMMANDINTERFACE_CC
#include "CommandInterface.cc"
#endif

#include <sstream>

namespace fatalmind {
namespace oracle {

template<class TM>
SQLSelectRow<TM>::SQLSelectRow(const std::string& SQL) 
: CommandCommon<TM>(SQL)
{
}

template<class TM>
CommandCommon<TM>* SQLSelectRow<TM>::clone() const {
	return new SQLSelectRow<TM>(*this);
}

template<class TM>
SQLSelectRow<TM>::~SQLSelectRow()
{
}

sb4 cdf_fetch_buffer(dvoid *vdmd
	, OCIDefine *defnp
	, ub4 iter
	, dvoid **bufpp
	, ub4 **alenpp
	, ub1 *piecep
	, dvoid **indpp
	, ub2 **rcpp
) {
	DefineMetaData* dmd = (DefineMetaData*)vdmd;
	if(piecep[0]!=(ub1)OCI_FIRST_PIECE) {
		dmd->done(**alenpp, *bufpp);
	}
	*bufpp = dmd->getDataPtr();
	dmd->resetSize();
	*alenpp = (ub4*)dmd->getSizePtr();

	*rcpp = 0;
	return OCI_CONTINUE;
}

void oracleBindOut::real_bindout(oracleHandles& hand, const OCIStatementHandle& sth, const std::string& SQL) {
	sword ret;
	_outparamt::const_iterator end = _outparam.end();
	int pos = 0;
	for (_outparamt::const_iterator i = _outparam.begin(); i != end; ++i) {
		++pos;
		if ((*i).get() != 0) {
			OCIDefine *defhp = NULL;
			DefineMetaData* dmd = _slb->getDefineMetaData(pos-1);
			bool buffered = !(dmd == 0);
			ub2*  sizeptr = 0;
			void* dataptr = 0;
			sb4*  indptr  = 0;
			ub4   size    = 0;
			ub4   mode    = OCI_DEFAULT;
			if (buffered) {
				const ColumnMetaData& cmd = _slmd->getColumnMetaData(pos-1);
				if (!cmd.piecewise()) {
					dataptr = dmd->getDataPtr();
					size    = cmd.getSize();
					indptr  = (sb4*)(*i)->indptr();
					sizeptr = (ub2*)dmd->getSizePtr();
				} else {
					size    = std::numeric_limits<sb4>::max();
					mode   |= OCI_DYNAMIC_FETCH;
				}
			} else {
				dataptr = (*i)->ptr();
				size    = (*i)->capacity();
				indptr  = (sb4*)(*i)->indptr();
			}
			if ((ret = OCIDefineByPos(sth.get()
				, &defhp
				, hand.getError()
				, pos
				, dataptr
				, size
				, (*i)->type()
				, indptr
				, sizeptr
				, (ub2 *) 0
				, mode
			)) != OCI_SUCCESS) {
				std::ostringstream os;
				os << "OCIDefineByPos(" << pos << ";" << (*i)->capacity() << ")";
				OracleExceptionFactory::throwit(os.str(), hand, ret, SQL);
			}
			if (_slmd->getColumnMetaData(pos-1).piecewise()) {
				if ((ret = OCIDefineDynamic(defhp
					, hand.getError()
					, dmd
					, cdf_fetch_buffer
				)) != OCI_SUCCESS) {
					OracleExceptionFactory::throwit("OCIDefineDynamic", hand, ret, SQL);
				}
			}
		} else {
			std::ostringstream os;
			os << "(OCIDefineByPos): bindout not called for column " << (pos -1);
			throw NFOOracleException(os.str(), 1008, SQL);
		}
	}
}

template<class TM>
bool SQLSelectRow<TM>::preSelectBuffer(oracleHandlesWithCache<TM>& hand)
{
	if (_slb.isNull()) {
		_slmd = hand.getSelectListMetaDataCache().get(_SQL);
		if (! _slmd.isNull()) {
			_slb.reset(new SelectListBuffer(_slmd, *this));
		}
	}
	return ! _slb.isNull();
}

template<class TM>
void SQLSelectRow<TM>::preSelectBufferBind(oracleHandles& hand)
{
	if (! _slb.isNull()) {
		real_bindout(hand, *_sth, _SQL);
	}
}

template<class TM>
void SQLSelectRow<TM>::postSelectBuffer(oracleHandlesWithCache<TM>& hand)
{
	if (_slb.isNull()) {
		_slmd = hand.getSelectListMetaDataCache().get(_SQL, SelectListMetaDataCtor(_SQL, *_sth, hand));
		_slb.reset(new SelectListBuffer(_slmd, *this));
		real_bindout(hand, *_sth, _SQL);
	}
}

template<class TM>
bool SQLSelectRow<TM>::prepare_buffer(oracleHandlesWithCache<TM>& hand, StatementCache& sthcache)
{
	bool exact = preSelectBuffer(hand);
	if (exact) {
		prepare(hand, sthcache, _slmd->getOraSQL());
	} else {
		prepare(hand, sthcache, oracleStatement(_SQL));
	}
	return exact;
}

template<class TM>
void SQLSelectRow<TM>::_execute(oracleHandlesWithCache<TM>& hand, StatementCache& sthcache) 
{
	sword ret;
	bool exact = prepare_buffer(hand, sthcache);

	real_bind(hand);
	preSelectBufferBind(hand);

	if (exact) {
		_start();
	}

	/* execute statement */
	if ((ret = OCIStmtExecute(hand.getSvcCtx()
		, _sth->get()
		, hand.getError()
		, (ub4) (exact? 1 : 0)
		, (ub4) 0
		, (CONST OCISnapshot *) 0
		, (OCISnapshot *) 0
		, (ub4) (exact ? OCI_EXACT_FETCH : OCI_DEFAULT) | super::_ac_mode())
	) != OCI_SUCCESS) {
		OracleExceptionFactory::throwit("OCIStmtExecute", hand, ret, _SQL);
	}

	postSelectBuffer(hand);	

	if (!exact) {
		internal::registerFinish fin(hand, *this, *_sth, _SQL);
		if (!fetch(hand, *_sth, _SQL)) {
			throw NFOOracleException("OCI_NO_DATA", 999999, _SQL);
		}

		if (_fetch(hand, *_sth, _SQL)) {
			throw NFOOracleException("ORA-01422: exact fetch returns more than requested number of rows", 1422, _SQL);
		}
	} else {
		_done();
	}
}

void oracleBindOut::_start() {
	_outparamt::const_iterator end = _outparam.end();

	for (_outparamt::const_iterator i = _outparam.begin()
		; i != end
		; ++i
	) {
		if ((*i).get() != 0) {
			(*i)->start();
		}
	}
}

void oracleBindOut::_done() {
	_outparamt::const_iterator end = _outparam.end();
	int pos = 0;
	for (_outparamt::const_iterator i = _outparam.begin()
		; i != end
		; ++i, ++pos
	) {
		DefineMetaData* dmd = _slb->getDefineMetaData(pos);
		if ((*i).get() != 0 && (dmd != 0)) {
			(*i)->done(dmd->getSize(), dmd->getDataPtr());
		}
	}
}


bool oracleBindOut::_fetch(oracleHandles& hand
	, const OCIStatementHandle& sth
	, const std::string& SQL
) {
	sword ret;
	if ((ret = OCIStmtFetch2( sth.get()
		, hand.getError()
		, 1
		, OCI_DEFAULT
		, 0
		, OCI_DEFAULT)
	) != OCI_SUCCESS) {
		if (ret == OCI_NO_DATA) {
			return false;
		} else {
			OracleExceptionFactory::throwit("OCIStmtFetch2", hand, ret, SQL);
		}
	}
	return true;
}

bool oracleBindOut::fetch(oracleHandles& hand
	, const OCIStatementHandle& sth
	, const std::string& SQL
) {
	_start();
	bool rv = _fetch(hand, sth, SQL);
	_done();
	return rv;
}

void oracleBindOut::finish(oracleHandles& hand
	, const OCIStatementHandle& sth
	, const std::string& SQL
) {
	sword ret;
	if ((ret = OCIStmtFetch2( sth.get()
		, hand.getError()
		, 0
		, OCI_DEFAULT
		, 0
		, OCI_DEFAULT)
	) != OCI_SUCCESS) {
		if (ret != OCI_NO_DATA) {
			OracleExceptionFactory::throwit("OCIStmtFetch2(finish)", hand, ret, SQL);
		}
	}
}

template class
SQL::BindOut<oracle::oracleRefType, oracle::oracleRefTypeImpl>;

template class
SQLSelectRow<DefaultThreadedModel>;

} // namespace oracle
} // namespace fatalmind
