//********************************************************************
//*** ResourcePool/oracle/SQLSelectRow.t.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelectRow.t.cc,v 1.16 2005/01/06 16:28:00 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "ResourcePool/oracle/ResourcePool.hh"
#include "t/main.hh"

#include "testconfig.cc"
#include "../SQL/SQLSelectRow.t.cc"

#include "cache/Cache.hh"
#include "Factory.cc"
#include "Resource.cc"
#include "ResourcePool/ResourcePool.cc"
#include "ResourcePool/Execute.cc"
#include "ResourcePool/Command.cc"
#include "SelectListMetaDataCache.hh"

void INIT_oracle_SQLSelectRow()
{
	test_SQLSelectRow_init();
}

void TEST_oracle_SQLSelectRow_one()
{
	test_SQLSelectRow_one();
}

void TEST_PERF_oracle_SQLSelectRow_one_spec_outer_C() {
	test_perf_SQLSelectRow_one_spec_outer();
}

typedef ResourcePool<ResourceType::oracleNopCache> RPNC;

void TEST_PERF_oracle_SQLSelectRow_one_spec_outer_NC()
{
	RPNC::Factory f(getService(), getUser(), getPass());
	RPNC rp(f);
	RPNC::Command::SQLSelectRow cmd("select id, ukn from hirsch where id = ?");
	int col1;
	std::string col2;

	cmd.bindout(0, col1);
	cmd.bindout(1, col2);
	for (int i = 0; i < 600; ++i) {
		cmd.bind(0, i);
		rp.execute(cmd);

		assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
		assertEqual(col1, i);
	}
}


void TEST_PERF_oracle_SQLSelectRow_one_spec_inner_C() {
	test_perf_SQLSelectRow_one_spec_inner();
}

void TEST_PERF_oracle_SQLSelectRow_one_spec_inner_NC()
{
	RPNC::Factory f(getService(), getUser(), getPass());
	RPNC rp(f);
    int col1;
    std::string col2;

    for (int i = 0; i < 600; ++i) {
        RPNC::Command::SQLSelectRow cmd("select id, ukn from hirsch where id = ?");
        cmd.bindout(0, col1);
        cmd.bindout(1, col2);

        cmd.bind(0, i);
        rp.execute(cmd);

        assertEqualT(std::string, col2, "hohoho ' kasdfjs \" sdf");
        assertEqual(col1, i);
    }
}


void TEST_PERF_oracle_SQLSelectRow_first_one()
{
	test_perf_SQLSelectRow_first_one();
}

void CLEANUP_oracle_SQLSelectRow() {
	test_SQLSelectRow_cleanup();
}

